#extension GL_EXT_shader_texture_lod : enable
#extension GL_OES_standard_derivatives : enable

precision highp float;

// Input variables for WebGL 1.0
varying vec3 v_position;
varying vec4 v_uv;
// Position in directional light space
varying vec4 v_PositionFromLight;

// Uniforms properties
uniform vec4 _Color;
uniform sampler2D _MainTex;
uniform sampler2D _FadeTex;
// Shadow specific uniforms
uniform sampler2D shadow0Sampler;
uniform float shadowDarkness;
uniform float shadowsEnabled;

float unpack(vec4 color)
{
    const vec4 bit_shift = vec4(1.0 / (255.0 * 255.0 * 255.0), 1.0 / (255.0 * 255.0), 1.0 / 255.0, 1.0);
    return dot(color, bit_shift);
}

float computeShadow(vec4 v_PositionFromLight, sampler2D shadowSampler, float darkness, float shadowsEnabled)
{
    if(shadowsEnabled == 0.0)
    {
        return 1.0;
    }
    
    vec3 depth = v_PositionFromLight.xyz / v_PositionFromLight.w;
    depth = 0.5 * depth + vec3(0.5);
    vec2 uv = depth.xy;

    if (uv.x < 0. || uv.x > 1.0 || uv.y < 0. || uv.y > 1.0)
    {
        return 1.0;
    }

    #ifndef SHADOWFULLFLOAT
        float shadow = unpack(texture2D(shadowSampler, uv));
    #else
        float shadow = texture2D(shadowSampler, uv).x;
    #endif

    if (depth.z > shadow)
    {
        return 1.0 - darkness;
    }

    return 1.0;
}

void main() {

    float fade = texture2D(_FadeTex, v_uv.xy).r;
    float shadowTerm = computeShadow(v_PositionFromLight, shadow0Sampler, shadowDarkness, shadowsEnabled);
    vec3 col = texture2D(_MainTex, v_uv.zw).rgb * shadowTerm;
    vec4 col4 = vec4(col, 1.0);
    gl_FragColor = mix(_Color, col4, fade);
}
